
// if the browser is successful in connecting to your MIDI devices, it will return a MIDIAccess object as an argument to the success callback
/**
Secure origin is any page that follows one of the following formats:
(https, *, *)
(wss, *, *)
(*, localhost, *)
(*, 127/8, *)
(*, ::1/128, *)
(file, *, — )
(chrome-extension, *, — )

// Chrome: set here chrome://settings/content/siteDetails?site=http%3A%2F%2Flocalhost
// Chromium is ok
// chromium https://chromium.woolyss.com/
//https://www.smashingmagazine.com/2018/03/web-midi-api/
**/

var selectMIDIIn = null;
var selectMIDIOut = null;
var midiAccess = null;
var midiIn = null;
var midiOut = null;
var outputIsLivid = false; // if using Livid Instruments CNTRLR

function onMIDIInit(midi) {

	// bail out	
	//console.table(midi);
	//onMIDISystemError();
	//if(midi.length=='undefined')	alert("error"); //onMIDISystemError();
  
	midiAccess = midi;
	//console.log(midiAccess.length);
	//if(midiAccess.length=='undefined')	alert("error"); //onMIDISystemError();

	// set reference to browser object for midi inputs
	selectMIDIIn=document.getElementById("midiIn");
	// initilize MIDI input select
	selectMIDIIn.options.length = 0;

	// set reference to browser object for midi outputs
	selectMIDIOut=document.getElementById("midiOut");
	// initilize MIDI output select
	selectMIDIOut.options.length = 0;

	var inputs;	
	var outputs;  

	if ((typeof(midiAccess.inputs) == "function")) {  //Old Skool MIDI inputs() code
  
		// check inputs 
		var inputs=midiAccess.inputs();
		for (var i=0; i<inputs.length; i++) {
			var str=inputs[i].name.toString();
			if (!midiIn && (str.indexOf("Controls") != -1)) {
				selectMIDIIn.appendChild(new Option(inputs[i].name,inputs[i].id,true,true));
				midiIn = inputs[i];
			} else
				selectMIDIIn.appendChild(new Option(inputs[i].name,inputs[i].id,false,false));
		}
		if (!midiIn)     midiIn = inputs[0];
		midiIn.onmidimessage = midiMessageReceived;

		// check outputs 
		outputs=midiAccess.outputs();
		for (var i=0; i<outputs.length; i++) {
			var str=outputs[i].name.toString();
			//if (!midiOut && (str.indexOf("Controls") != -1)) {
			// set our preferred USB MIDI Interface output
			if (!midiOut && (str.indexOf("Interface") != -1)) {
				selectMIDIOut.appendChild(new Option(outputs[i].name,outputs[i].id,true,true));
				midiOut = outputs[i];
			} else
				selectMIDIOut.appendChild(new Option(outputs[i].name,outputs[i].id,false,false));
			}
			if (!midiOut)      midiOut = outputs[0];
  
	} else {  // new MIDIMap implementation:
  
		// check inputs 
		inputs=midiAccess.inputs.values();
		for ( var input = inputs.next(); input && !input.done; input = inputs.next()) {
			input = input.value;
			var str=input.name.toString();
			//var preferred = !midiIn && (str.indexOf("Controls") != -1);
			// set our preferred USB MIDI Interface input
			var preferred = !midiIn && (str.indexOf("Interface") != -1);
			selectMIDIIn.appendChild(new Option(input.name,input.id,preferred,preferred));
			if (preferred) {
				midiIn = input;
				midiIn.onmidimessage = midiMessageReceived;
			}
		}
		if (!midiIn) {
			// bail out	
			if(!midiAccess.inputs.values().next().value) {
				onMIDISystemError();
				return; 	
			}	
			midiIn = midiAccess.inputs.values().next().value;
			midiIn.onmidimessage = midiMessageReceived;
		}
	
		// check outputs 
		outputs=midiAccess.outputs.values();
		for ( var output = outputs.next(); output && !output.done; output = outputs.next()) {
			output = output.value;
			var str=output.name.toString();
			//var preferred = !midiOut && (str.indexOf("Controls") != -1);
			// set our preferred USB MIDI Interface output
			var preferred = !midiOut && (str.indexOf("Interface") != -1);
			//var preferred = !midiOut && (str.indexOf("UM-1") != -1);
			selectMIDIOut.appendChild(new Option(output.name,output.id,preferred,preferred));
			if (preferred) {
				midiOut = output;
				outputIsLivid = true;
			}
		}
		if (!midiOut) {
			// bail out	
			if(!midiAccess.outputs.values().next().value) {
				onMIDISystemError();
				return; 	
			}
			midiOut = midiAccess.outputs.values().next().value;
		}
	}

	// no midi in needed - except if we use keyboard as controller 	
	selectMIDIIn.onchange = changeMIDIIn;
	// midi out 	
	selectMIDIOut.onchange = changeMIDIOut;

	if( (!midiIn && !midiOut)) document.getElementById("notice").innerHTML += "<span class='unsupported'>No devices found. </span>";
	else document.getElementById("notice").innerHTML += "<span class='supported'>Devices are available. </span>";

	// we have only one midi device
	setActiveInstrument(0);	
	updateActiveInstruments();
  
	/**
	// Livid controller custom commands
	if (midiOut && outputIsLivid) {
		// light up the play button
		midiOut.send( [0x90, 3, 32] );
		// turn off the stop button
		midiOut.send( [0x80, 7, 1] );
	}
	**/
}

function changeMIDIOut( ev ) {
	var id = ev.target[ev.target.selectedIndex].value;

	if ((typeof(midiAccess.outputs) == "function"))   //Old Skool MIDI outputs() code
		midiOut = midiAccess.outputs()[ev.target.selectedIndex];
	else
		midiOut = midiAccess.outputs.get(id);

	/**	
	// Livid controller custom commands
	outputIsLivid = midiOut.name.indexOf("Controls") != -1;
	if (midiOut  && outputIsLivid) {
		updateActiveInstruments();
		// light up the play button
		midiOut.send( [0x90, 3, 32] );
		// turn off the stop button
		midiOut.send( [0x80, 7, 1] );
	}
	**/ 
}

function noteOn( noteNumber, velocity) {
	// we have 6 diaprojectors with at least 50 frames
	// noteNumber (1-6) is diaprojector	// maybe a channel?
	// velocity > 0 = trigger frame change
	// on channel 0
    if(midiOut) midiOut.send( [0x90, noteNumber, 1] ); // note on -> veocity > 0
    //if(midiOut) midiOut.send( [0x90, noteNumber, 0] ); // note off -> velocity 0
	// or 
    //if(midiOut) midiOut.send( [0x80, 7, 1] ); // real note off
}

function noteOff( noteNumber ) {
	// no need
    if(midiOut != null) midiOut.send( [0x90, noteNumber, 0] ); // note off -> velocity 0
	// or 
    //if(midiOut) midiOut.send( [0x80, noteNumber, 1] ); // real note off
}


var currentlyActiveInstrument = 0;
var instrumentActive = [true,true,true,true,true,true];

function updateActiveInstruments() {
  if (!midiOut)    return;

  for (var i=0;i<6; i++) {
    if (instrumentActive[i])     midiOut.send( [0x90, keyForInstrument(i)+8, colorForIntrument(i)] );
    else      midiOut.send( [0x80, keyForInstrument(i)+8, 0x00] );
  }	
}
function setActiveInstrument(index) {
  //turn off the last lit-up instrument
  //if (midiOut&&outputIsLivid)    midiOut.send( [0x80, keyForInstrument(currentlyActiveInstrument), 0x00] );

  currentlyActiveInstrument = index;

  // turn on the new instrument button
  //if (midiOut&&outputIsLivid)    midiOut.send( [0x90, keyForInstrument(index), colorForIntrument(index)] );

}

// maybe use this for keyboard control
function changeMIDIIn( ev ) {
  if (midiIn)    midiIn.onmidimessage = null;
  var id = ev.target[ev.target.selectedIndex].value;
  if ((typeof(midiAccess.inputs) == "function"))   //Old Skool MIDI inputs() code
    midiIn = midiAccess.inputs()[ev.target.selectedIndex];
  else    midiIn = midiAccess.inputs.get(id);
  if (midiIn)    midiIn.onmidimessage = midiMessageReceived;
}

// maybe use this for keyboard control
function midiMessageReceived( ev ) {
    var cmd = ev.data[0] >> 4;
    var channel = ev.data[0] & 0xf;
    var noteNumber = ev.data[1];
    var velocity = 0;
    if (ev.data.length > 2)  velocity = ev.data[2];

    if ( cmd==8 || ((cmd==9)&&(velocity==0)) ) { // with MIDI, note on with velocity zero is the same as note off
      // note off
      if (channel!=9)   noteOff( noteNumber );
    } else if (cmd == 9) {
      // note on
      console.log("Note on - channel: " + channel + " note: " + noteNumber + " velocity: " + velocity );
      //if ((channel == 9)||(channel==1)) playDrum(noteNumber, velocity);
      //else
		  noteOn( noteNumber, velocity);
    } else if ((ev.data.length == 6) &&
      (ev.data[0] == 0xf0) &&
      (ev.data[1] == 0x7f) &&
      (ev.data[3] == 0x06) &&
      (ev.data[5] == 0xf7) ) { // MIDI Machine Control (MMC) message
      switch (ev.data[4]) {
        case 0x01: // stop
          //handleStop();
          break;
        case 0x02: // start
          //handlePlay();
          break;
      }
    } else {
      console.log("unrecognized message");
    }
}

function keyForInstrument(index) {
    if (index <3)
      return index;
    else
      return index+1;
}

function colorForIntrument(index) {
  if (index<0 || index>6)
    return 0;

  if (index == 1)
    return 64;
  else
    return Math.pow(2,index);
}
